#!/bin/bash
# flac2whatever <ed@irc.rizon.net> GLWTPL

set -e

webroot=.
[ -z "$1" ] ||
	webroot="$1"

logfile="$webroot/.transcode.log.$(date +%Y-%m%d-%H%M%S)"
echo hi > $logfile
chmod 600 $logfile

{
cat <<'EOF'

# list of formats to encode:
# 1) file extension
# 2) output directory
# 3) ffmpeg arguments

# <TARGET FORMAT LIST BEGIN>

# ogg ../vorbis-q4 -acodec libvorbis -q:a 4
# mp3 ../mp3-v2    -acodec libmp3lame -q:a 2 -q 0
opus .opus -acodec libopus -b:a 128k
# opus .opush -acodec libopus -b:a 224k

# <TARGET FORMAT LIST END>
EOF
} |
grep -F codec |
grep -vF '#' |
while IFS=' ' read -r outext outbase ffargs
do
	cores=4
	jctl=$(mktemp -d)
	while IFS= read -r flacpath
	do
		while true
		do
			bfiles=($jctl/*)
			busy=${#bfiles[@]}
			[ $busy -lt $cores ] &&
				break
			sleep 0.1
		done
		flacfile="${flacpath##*/}"
		fn="${flacfile%.*}"
		dir="${flacpath%/*}"
		dir="${dir:2}"
		outfile="$webroot/$outbase/$dir/$fn.$outext"

		ageflac=$(date +%s -r "$flacpath")
		ageout=$(date +%s -r "$outfile" 2>/dev/null || echo 0)

		now=$(date +%s)
		agedelta=$((now-ageflac))
		[ $agedelta -gt 600 ] ||
		{
			printf 'flac too recent; skipping %s\n' "$flacpath" | tee -a $logfile
			continue
		}
		
		#printf '\033[36m%s \033[35m(%s)\033[0m => \033[35m(%s) \033[34m%s\033[1;33m/\033[0;34m%s\033[1;33m/\033[0;34m%s\033[1;33m.\033[0;34m%s\033[0m ... ' "$flacpath" "$ageflac" "$ageout" "$outbase" "$dir" "$fn" "$outext"; echo; done
		
		[ $ageflac -gt $ageout ] ||
			continue
		
		mkdir -p "$webroot/$outbase/$dir"
		
		printf '\033[36m%s\033[0m\n' "$flacpath"
		me=$jctl/$(date +%s.%N)
		touch $me
		{
			nice -n 20 ionice -c 3 \
			ffmpeg -hide_banner -loglevel warning -y -i "$flacpath" -map 0:a $ffargs "$outfile" < /dev/null

			rm $me
		} &
	done < <(
		find -iname \*.flac |
		sort
	)

	now=$(date +%s)
	find -type f -mindepth 2 |
	grep -aiE '/(cd|cover|folder)\.(jpg|jpeg|png|gif)$' |
	sort |
	while IFS= read -r coverpath
	do
		dir="${coverpath%/*}"
		dir="${dir:2}"
		coverfile="${coverpath##*/}"
		outfile="$webroot/$outbase/$dir/$coverfile"

		ageorig=$(date +%s -r "$coverpath")
		ageout=$(date +%s -r "$outfile" 2>/dev/null || echo 0)
		agedelta=$((now-ageorig))
		[ $agedelta -gt 600 ] ||
		{
			printf 'cover too recent; skipping %s\n' "$coverpath" | tee -a $logfile
			continue
		}

		[ $ageorig -gt $ageout ] ||
			continue

		# folder creation should occur in the transcoding loop
		# however if the flac is close to 10 minutes old it might get skipped
		# so consider any error non-critical
		cp -pv "$coverpath" "$outfile" || true
	done
done

echo
echo "finished; see $logfile"
ls -al $logfile || true

